@@ -29,7 +29,6 @@
 int RemoteTaskCheckInterval = 100; /* per cycle sleep interval in millisecs */
 int TaskExecutorType = MULTI_EXECUTOR_REAL_TIME; /* distributed executor type */
 bool BinaryMasterCopyFormat = false; /* copy data from workers in binary format */
-int MultiTaskQueryLogLevel = MULTI_TASK_QUERY_INFO_OFF; /* multi-task query log level */
 
 
 /*
@@ -62,15 +61,6 @@ JobExecutorType(MultiPlan *multiPlan)
 		return MULTI_EXECUTOR_COORDINATOR_INSERT_SELECT;
 	}
 
-	/* if it is not a router executable plan, inform user according to the log level */
-	if (MultiTaskQueryLogLevel != MULTI_TASK_QUERY_INFO_OFF)
-	{
-		ereport(MultiTaskQueryLogLevel, (errmsg("multi-task query about to be executed"),
-										 errhint("Queries are split to multiple tasks "
-												 "if they have to be split into several"
-												 " queries on the workers.")));
-	}
-
 	Assert(multiPlan->operation == CMD_SELECT);
 
 	workerNodeList = ActiveReadableNodeList();
