@@ -33,6 +33,7 @@
 #include "distributed/multi_server_executor.h"
 #include "distributed/pg_dist_partition.h"
 #include "distributed/worker_protocol.h"
+#include "distributed/version_compat.h"
 #include "storage/fd.h"
 #include "utils/builtins.h"
 #include "utils/hsearch.h"
@@ -1349,7 +1350,7 @@ ManageTransmitExecution(TaskTracker *transmitTracker,
 				int fileFlags = (O_APPEND | O_CREAT | O_RDWR | O_TRUNC | PG_BINARY);
 				int fileMode = (S_IRUSR | S_IWUSR);
 
-				int32 fileDescriptor = BasicOpenFile(filename, fileFlags, fileMode);
+				int32 fileDescriptor = BasicOpenFilePerm(filename, fileFlags, fileMode);
 				if (fileDescriptor >= 0)
 				{
 					/*
