@@ -35,9 +35,11 @@
 #include "distributed/metadata_cache.h"
 #include "distributed/metadata_sync.h"
 #include "distributed/multi_join_order.h"
+#include "distributed/multi_partitioning_utils.h"
 #include "distributed/pg_dist_node.h"
 #include "distributed/worker_manager.h"
 #include "distributed/worker_transaction.h"
+#include "distributed/version_compat.h"
 #include "foreign/foreign.h"
 #include "nodes/pg_list.h"
 #include "utils/builtins.h"
@@ -250,6 +252,16 @@ MetadataCreateCommands(void)
 		if (ShouldSyncTableMetadata(cacheEntry->relationId))
 		{
 			propagatedTableList = lappend(propagatedTableList, cacheEntry);
+
+			if (PartitionedTable(cacheEntry->relationId))
+			{
+				char *relationName = get_rel_name(cacheEntry->relationId);
+
+				ereport(ERROR, (errcode(ERRCODE_FEATURE_NOT_SUPPORTED),
+								errmsg("cannot perform metadata sync for "
+									   "partitioned table \"%s\"",
+									   relationName)));
+			}
 		}
 	}
 
@@ -984,7 +996,7 @@ TypeOfColumn(Oid tableId, int16 columnId)
 {
 	Relation tableRelation = relation_open(tableId, NoLock);
 	TupleDesc tupleDescriptor = RelationGetDescr(tableRelation);
-	Form_pg_attribute attrForm = tupleDescriptor->attrs[columnId - 1];
+	Form_pg_attribute attrForm = TupleDescAttr(tupleDescriptor, columnId - 1);
 	relation_close(tableRelation, NoLock);
 	return attrForm->atttypid;
 }
