@@ -30,6 +30,7 @@
 #include "optimizer/clauses.h"
 #include "optimizer/planner.h"
 #include "optimizer/restrictinfo.h"
+#include "optimizer/tlist.h"
 #include "optimizer/var.h"
 #include "parser/parsetree.h"
 #include "parser/parse_coerce.h"
@@ -53,6 +54,7 @@ static DeferredErrorMessage * DistributedInsertSelectSupported(Query *queryTree,
 															   RangeTblEntry *subqueryRte,
 															   bool allReferenceTables);
 static DeferredErrorMessage * MultiTaskRouterSelectQuerySupported(Query *query);
+static bool HasUnsupportedDistinctOn(Query *query);
 static DeferredErrorMessage * InsertPartitionColumnMatchesSelect(Query *query,
 																 RangeTblEntry *insertRte,
 																 RangeTblEntry *
@@ -62,8 +64,7 @@ static DeferredErrorMessage * InsertPartitionColumnMatchesSelect(Query *query,
 static MultiPlan * CreateCoordinatorInsertSelectPlan(Query *parse);
 static DeferredErrorMessage * CoordinatorInsertSelectSupported(Query *insertSelectQuery);
 static Query * WrapSubquery(Query *subquery);
-static void CastSelectTargetList(List *selectTargetList, Oid targetRelationId,
-								 List *insertTargetList);
+static bool CheckInsertSelectQuery(Query *query);
 
 
 /*
@@ -73,18 +74,62 @@ static void CastSelectTargetList(List *selectTargetList, Oid targetRelationId,
  *
  * Note that the input query should be the original parsetree of
  * the query (i.e., not passed trough the standard planner).
+ */
+bool
+InsertSelectIntoDistributedTable(Query *query)
+{
+	bool insertSelectQuery = CheckInsertSelectQuery(query);
+
+	if (insertSelectQuery)
+	{
+		RangeTblEntry *insertRte = ExtractInsertRangeTableEntry(query);
+		if (IsDistributedTable(insertRte->relid))
+		{
+			return true;
+		}
+	}
+
+	return false;
+}
+
+
+/*
+ * InsertSelectIntoLocalTable checks whether INSERT INTO ... SELECT inserts
+ * into local table. Note that query must be a sample of INSERT INTO ... SELECT
+ * type of query.
+ */
+bool
+InsertSelectIntoLocalTable(Query *query)
+{
+	bool insertSelectQuery = CheckInsertSelectQuery(query);
+
+	if (insertSelectQuery)
+	{
+		RangeTblEntry *insertRte = ExtractInsertRangeTableEntry(query);
+		if (!IsDistributedTable(insertRte->relid))
+		{
+			return true;
+		}
+	}
+
+	return false;
+}
+
+
+/*
+ * CheckInsertSelectQuery returns true when the input query is an INSERT INTO
+ * ... SELECT kind of query.
  *
  * This function is inspired from getInsertSelectQuery() on
  * rewrite/rewriteManip.c.
  */
-bool
-InsertSelectIntoDistributedTable(Query *query)
+static bool
+CheckInsertSelectQuery(Query *query)
 {
 	CmdType commandType = query->commandType;
 	List *fromList = NULL;
 	RangeTblRef *rangeTableReference = NULL;
 	RangeTblEntry *subqueryRte = NULL;
-	RangeTblEntry *insertRte = NULL;
 
 	if (commandType != CMD_INSERT)
 	{
@@ -117,12 +162,6 @@ InsertSelectIntoDistributedTable(Query *query)
 	/* ensure that there is a query */
 	Assert(IsA(subqueryRte->subquery, Query));
 
-	insertRte = ExtractInsertRangeTableEntry(query);
-	if (!IsDistributedTable(insertRte->relid))
-	{
-		return false;
-	}
-
 	return true;
 }
 
@@ -228,15 +267,6 @@ CreateDistributedInsertSelectPlan(Query *originalQuery,
 		++taskIdIndex;
 	}
 
-	if (MultiTaskQueryLogLevel != MULTI_TASK_QUERY_INFO_OFF &&
-		list_length(sqlTaskList) > 1)
-	{
-		ereport(MultiTaskQueryLogLevel, (errmsg("multi-task query about to be executed"),
-										 errhint("Queries are split to multiple tasks "
-												 "if they have to be split into several"
-												 " queries on the workers.")));
-	}
-
 	/* Create the worker job */
 	workerJob = CitusMakeNode(Job);
 	workerJob->taskList = sqlTaskList;
@@ -289,14 +319,6 @@ DistributedInsertSelectSupported(Query *queryTree, RangeTblEntry *insertRte,
 							 NULL, NULL);
 	}
 
-	if (GetLocalGroupId() != 0)
-	{
-		return DeferredError(ERRCODE_FEATURE_NOT_SUPPORTED,
-							 "distributed INSERT ... SELECT can only be performed from "
-							 "the coordinator",
-							 NULL, NULL);
-	}
-
 	/* we do not expect to see a view in modify target */
 	foreach(rangeTableCell, queryTree->rtable)
 	{
@@ -443,6 +465,7 @@ RouterModifyTaskForShardInterval(Query *originalQuery, ShardInterval *shardInter
 	List *shardOpExpressions = NIL;
 	RestrictInfo *shardRestrictionList = NULL;
 	DeferredErrorMessage *planningError = NULL;
+	bool multiShardModifyQuery = false;
 
 	/* grab shared metadata lock to stop concurrent placement additions */
 	LockShardDistributionMetadata(shardId, ShareLock);
@@ -502,7 +525,10 @@ RouterModifyTaskForShardInterval(Query *originalQuery, ShardInterval *shardInter
 	 */
 	planningError = PlanRouterQuery(copiedSubquery, copiedRestrictionContext,
 									&selectPlacementList, &selectAnchorShardId,
-									&relationShardList, replacePrunedQueryWithDummy);
+									&relationShardList, replacePrunedQueryWithDummy,
+									&multiShardModifyQuery);
+
+	Assert(!multiShardModifyQuery);
 
 	if (planningError)
 	{
@@ -737,6 +763,8 @@ MultiTaskRouterSelectQuerySupported(Query *query)
 {
 	List *queryList = NIL;
 	ListCell *queryCell = NULL;
+	StringInfo errorDetail = NULL;
+	bool hasUnsupportedDistinctOn = false;
 
 	ExtractQueryWalker((Node *) query, &queryList);
 	foreach(queryCell, queryList)
@@ -758,7 +786,7 @@ MultiTaskRouterSelectQuerySupported(Query *query)
 		if (subquery->limitCount != NULL)
 		{
 			return DeferredError(ERRCODE_FEATURE_NOT_SUPPORTED,
-								 "LIMIT clauses are not allowed in distirbuted INSERT "
+								 "LIMIT clauses are not allowed in distributed INSERT "
 								 "... SELECT queries",
 								 NULL, NULL);
 		}
@@ -772,17 +800,34 @@ MultiTaskRouterSelectQuerySupported(Query *query)
 								 NULL, NULL);
 		}
 
-		/*
-		 * We could potentially support window clauses where the data is partitioned
-		 * over distribution column. For simplicity, we currently do not support window
-		 * clauses at all.
-		 */
-		if (subquery->windowClause != NULL)
+		/* group clause list must include partition column */
+		if (subquery->groupClause)
 		{
-			return DeferredError(ERRCODE_FEATURE_NOT_SUPPORTED,
-								 "window functions are not allowed in distributed "
-								 "INSERT ... SELECT queries",
-								 NULL, NULL);
+			List *groupClauseList = subquery->groupClause;
+			List *targetEntryList = subquery->targetList;
+			List *groupTargetEntryList = GroupTargetEntryList(groupClauseList,
+															  targetEntryList);
+			bool groupOnPartitionColumn = TargetListOnPartitionColumn(subquery,
+																	  groupTargetEntryList);
+			if (!groupOnPartitionColumn)
+			{
+				return DeferredError(ERRCODE_FEATURE_NOT_SUPPORTED,
+									 "Group by list without distribution column is "
+									 "not allowed  in distributed INSERT ... "
+									 "SELECT queries",
+									 NULL, NULL);
+			}
+		}
+
+		/*
+		 * We support window functions when the window function
+		 * is partitioned on distribution column.
+		 */
+		if (subquery->windowClause && !SafeToPushdownWindowFunction(subquery,
+																	&errorDetail))
+		{
+			return DeferredError(ERRCODE_FEATURE_NOT_SUPPORTED, errorDetail->data, NULL,
+								 NULL);
 		}
 
 		if (subquery->setOperations != NULL)
@@ -808,14 +853,14 @@ MultiTaskRouterSelectQuerySupported(Query *query)
 		}
 
 		/*
-		 * We cannot support DISTINCT ON clauses since it could be on a non-partition column.
-		 * In that case, there is no way that Citus can support this.
+		 * We don't support DISTINCT ON clauses on non-partition columns.
 		 */
-		if (subquery->hasDistinctOn)
+		hasUnsupportedDistinctOn = HasUnsupportedDistinctOn(subquery);
+		if (hasUnsupportedDistinctOn)
 		{
 			return DeferredError(ERRCODE_FEATURE_NOT_SUPPORTED,
-								 "DISTINCT ON clauses are not allowed in distributed "
-								 "INSERT ... SELECT queries",
+								 "DISTINCT ON (non-partition column) clauses are not "
+								 "allowed in distributed INSERT ... SELECT queries",
 								 NULL, NULL);
 		}
 	}
@@ -824,6 +869,36 @@ MultiTaskRouterSelectQuerySupported(Query *query)
 }
 
 
+/*
+ * HasUnsupportedDistinctOn returns true if the query has distinct on and
+ * distinct targets do not contain partition column.
+ */
+static bool
+HasUnsupportedDistinctOn(Query *query)
+{
+	ListCell *distinctCell = NULL;
+
+	if (!query->hasDistinctOn)
+	{
+		return false;
+	}
+
+	foreach(distinctCell, query->distinctClause)
+	{
+		SortGroupClause *distinctClause = lfirst(distinctCell);
+		TargetEntry *distinctEntry = get_sortgroupclause_tle(distinctClause,
+															 query->targetList);
+
+		if (IsPartitionColumn(distinctEntry->expr, query))
+		{
+			return false;
+		}
+	}
+
+	return true;
+}
+
+
 /*
  * InsertPartitionColumnMatchesSelect returns NULL the partition column in the
  * table targeted by INSERTed matches with the any of the SELECTed table's
@@ -1108,10 +1183,6 @@ CreateCoordinatorInsertSelectPlan(Query *parse)
 
 	ReorderInsertSelectTargetLists(insertSelectQuery, insertRte, selectRte);
 
-	/* make sure the SELECT returns the right type for copying into the table */
-	CastSelectTargetList(selectQuery->targetList, targetRelationId,
-						 insertSelectQuery->targetList);
-
 	multiPlan->insertSelectSubquery = selectQuery;
 	multiPlan->insertTargetList = insertSelectQuery->targetList;
 	multiPlan->targetRelationId = targetRelationId;
@@ -1232,62 +1303,3 @@ WrapSubquery(Query *subquery)
 
 	return outerQuery;
 }
-
-
-/*
- * CastSelectTargetList adds casts to the target entries in selectTargetList
- * to match the type in insertTargetList. This ensures that the results of
- * the SELECT will have the right type when serialised during COPY. For
- * example, a float that is inserted into a an int column normally has an
- * implicit cast, but if we send it through the COPY protocol the serialised
- * form would contain decimal notation, which is not valid for int.
- */
-static void
-CastSelectTargetList(List *selectTargetList, Oid targetRelationId, List *insertTargetList)
-{
-	ListCell *insertTargetCell = NULL;
-	ListCell *selectTargetCell = NULL;
-
-	/* add casts when the SELECT output does not directly match the table */
-	forboth(insertTargetCell, insertTargetList,
-			selectTargetCell, selectTargetList)
-	{
-		TargetEntry *insertTargetEntry = (TargetEntry *) lfirst(insertTargetCell);
-		TargetEntry *selectTargetEntry = (TargetEntry *) lfirst(selectTargetCell);
-
-		Var *columnVar = NULL;
-		Oid columnType = InvalidOid;
-		int32 columnTypeMod = 0;
-		Oid selectOutputType = InvalidOid;
-
-		/* indirection is not supported, e.g. INSERT INTO table (composite_column.x) */
-		if (!IsA(insertTargetEntry->expr, Var))
-		{
-			ereport(ERROR, (errmsg("can only handle regular columns in the target "
-								   "list")));
-		}
-
-		columnVar = (Var *) insertTargetEntry->expr;
-		columnType = get_atttype(targetRelationId, columnVar->varattno);
-		columnTypeMod = get_atttypmod(targetRelationId, columnVar->varattno);
-		selectOutputType = columnVar->vartype;
-
-		/*
-		 * If the type in the target list does not match the type of the column,
-		 * we need to cast to the column type. PostgreSQL would do this
-		 * automatically during the insert, but we're passing the SELECT
-		 * output directly to COPY.
-		 */
-		if (columnType != selectOutputType)
-		{
-			Expr *selectExpression = selectTargetEntry->expr;
-			Expr *typeCastedSelectExpr =
-				(Expr *) coerce_to_target_type(NULL, (Node *) selectExpression,
-											   selectOutputType, columnType,
-											   columnTypeMod, COERCION_EXPLICIT,
-											   COERCE_IMPLICIT_CAST, -1);
-
-			selectTargetEntry->expr = typeCastedSelectExpr;
-		}
-	}
-}
