@@ -27,6 +27,7 @@
 #include "access/stratnum.h"
 #include "catalog/indexing.h"
 #include "catalog/namespace.h"
+#include "catalog/pg_class.h"
 #include "catalog/pg_constraint.h"
 #include "distributed/metadata_cache.h"
 #include "distributed/relay_utility.h"
@@ -99,8 +100,19 @@ RelayEventExtendNames(Node *parseTree, char *schemaName, uint64 shardId)
 			{
 				AlterTableCmd *command = (AlterTableCmd *) lfirst(commandCell);
 
-				if (command->subtype == AT_AddConstraint ||
-					command->subtype == AT_DropConstraint)
+				if (command->subtype == AT_AddConstraint)
+				{
+					Constraint *constraint = (Constraint *) command->def;
+
+					if (constraint->contype == CONSTR_PRIMARY && constraint->indexname)
+					{
+						char **indexName = &(constraint->indexname);
+						AppendShardIdToName(indexName, shardId);
+					}
+
+					AppendShardIdToConstraintName(command, shardId);
+				}
+				if (command->subtype == AT_DropConstraint)
 				{
 					AppendShardIdToConstraintName(command, shardId);
 				}
@@ -109,6 +121,17 @@ RelayEventExtendNames(Node *parseTree, char *schemaName, uint64 shardId)
 					char **indexName = &(command->name);
 					AppendShardIdToName(indexName, shardId);
 				}
+				else if (command->subtype == AT_ReplicaIdentity)
+				{
+					ReplicaIdentityStmt *replicaIdentity =
+						(ReplicaIdentityStmt *) command->def;
+
+					if (replicaIdentity->identity_type == REPLICA_IDENTITY_INDEX)
+					{
+						char **indexName = &(replicaIdentity->name);
+						AppendShardIdToName(indexName, shardId);
+					}
+				}
 			}
 
 			break;
@@ -708,7 +731,15 @@ shard_name(PG_FUNCTION_ARGS)
 
 	schemaId = get_rel_namespace(relationId);
 	schemaName = get_namespace_name(schemaId);
-	qualifiedName = quote_qualified_identifier(schemaName, relationName);
+
+	if (strncmp(schemaName, "public", NAMEDATALEN) == 0)
+	{
+		qualifiedName = (char *) quote_identifier(relationName);
+	}
+	else
+	{
+		qualifiedName = quote_qualified_identifier(schemaName, relationName);
+	}
 
 	PG_RETURN_TEXT_P(cstring_to_text(qualifiedName));
 }
