@@ -22,7 +22,6 @@
 #include "distributed/backend_data.h"
 #include "distributed/citus_nodefuncs.h"
 #include "distributed/connection_management.h"
-#include "distributed/connection_management.h"
 #include "distributed/distributed_deadlock_detection.h"
 #include "distributed/maintenanced.h"
 #include "distributed/master_metadata_utility.h"
@@ -41,6 +40,7 @@
 #include "distributed/placement_connection.h"
 #include "distributed/remote_commands.h"
 #include "distributed/shared_library_init.h"
+#include "distributed/statistics_collection.h"
 #include "distributed/task_tracker.h"
 #include "distributed/transaction_management.h"
 #include "distributed/worker_manager.h"
@@ -65,6 +65,8 @@ static void WarningForEnableDeadlockPrevention(bool newval, void *extra);
 static bool ErrorIfNotASuitableDeadlockFactor(double *newval, void **extra,
 											  GucSource source);
 static void NormalizeWorkerListPath(void);
+static bool StatisticsCollectionGucCheckHook(bool *newval, void **extra, GucSource
+											 source);
 
 
 /* *INDENT-OFF* */
@@ -107,6 +109,16 @@ static const struct config_enum_entry multi_shard_commit_protocol_options[] = {
 	{ NULL, 0, false }
 };
 
+static const struct config_enum_entry citus_ssl_mode_options[] = {
+	{ "disable", CITUS_SSL_MODE_DISABLE, false },
+	{ "allow", CITUS_SSL_MODE_ALLOW, false },
+	{ "prefer", CITUS_SSL_MODE_PREFER, false },
+	{ "require", CITUS_SSL_MODE_REQUIRE, false },
+	{ "verify-ca", CITUS_SSL_MODE_VERIFY_CA, false },
+	{ "verify-full", CITUS_SSL_MODE_VERIFY_FULL, false },
+	{ NULL, 0, false }
+};
+
 static const struct config_enum_entry multi_task_query_log_level_options[] = {
 	{ "off", MULTI_TASK_QUERY_INFO_OFF, false },
 	{ "debug", DEBUG2, false },
@@ -117,6 +129,12 @@ static const struct config_enum_entry multi_task_query_log_level_options[] = {
 	{ NULL, 0, false }
 };
 
+static const struct config_enum_entry multi_shard_modify_connection_options[] = {
+	{ "parallel", PARALLEL_CONNECTION, false },
+	{ "sequential", SEQUENTIAL_CONNECTION, false },
+	{ NULL, 0, false }
+};
+
 /* *INDENT-ON* */
 
 
@@ -217,7 +235,7 @@ multi_log_hook(ErrorData *edata)
 		MyBackendGotCancelledDueToDeadlock())
 	{
 		edata->sqlerrcode = ERRCODE_T_R_DEADLOCK_DETECTED;
-		edata->message = "canceling the transaction since it has "
+		edata->message = "canceling the transaction since it was "
 						 "involved in a distributed deadlock";
 	}
 }
@@ -295,6 +313,19 @@ RegisterCitusConfigVariables(void)
 		NULL, NULL, NULL);
 	NormalizeWorkerListPath();
 
+	DefineCustomEnumVariable(
+		"citus.sslmode",
+		gettext_noop("SSL mode to use for connections to worker nodes."),
+		gettext_noop("When connecting to a worker node, specify whether the SSL mode"
+					 "mode for the connection is 'disable', 'allow', 'prefer' "
+					 "(the default), 'require', 'verify-ca' or 'verify-full'."),
+		&CitusSSLMode,
+		CITUS_SSL_MODE_PREFER,
+		citus_ssl_mode_options,
+		PGC_POSTMASTER,
+		GUC_SUPERUSER_ONLY,
+		NULL, NULL, NULL);
+
 	DefineCustomBoolVariable(
 		"citus.binary_master_copy_format",
 		gettext_noop("Use the binary master copy format."),
@@ -730,6 +761,16 @@ RegisterCitusConfigVariables(void)
 		0,
 		NULL, NULL, NULL);
 
+	DefineCustomEnumVariable(
+		"citus.multi_shard_modify_mode",
+		gettext_noop("Sets the connection type for multi shard modify queries"),
+		NULL,
+		&MultiShardConnectionType,
+		PARALLEL_CONNECTION, multi_shard_modify_connection_options,
+		PGC_USERSET,
+		0,
+		NULL, NULL, NULL);
+
 	DefineCustomStringVariable(
 		"citus.version",
 		gettext_noop("Shows the Citus library version"),
@@ -785,6 +826,24 @@ RegisterCitusConfigVariables(void)
 		0,
 		NULL, NULL, NULL);
 
+	DefineCustomBoolVariable(
+		"citus.enable_statistics_collection",
+		gettext_noop("Enables sending basic usage statistics to Citus."),
+		gettext_noop("Citus uploads daily anonymous usage reports containing "
+					 "rounded node count, shard size, distributed table count, "
+					 "and operating system name. This configuration value controls "
+					 "whether these reports are sent."),
+		&EnableStatisticsCollection,
+#ifdef HAVE_LIBCURL
+		true,
+#else
+		false,
+#endif
+		PGC_SIGHUP,
+		GUC_SUPERUSER_ONLY | GUC_NO_SHOW_ALL,
+		&StatisticsCollectionGucCheckHook,
+		NULL, NULL);
+
 	/* warn about config items in the citus namespace that are not registered above */
 	EmitWarningsOnPlaceholders("citus");
 }
@@ -866,3 +925,25 @@ NormalizeWorkerListPath(void)
 					PGC_S_OVERRIDE);
 	free(absoluteFileName);
 }
+
+
+static bool
+StatisticsCollectionGucCheckHook(bool *newval, void **extra, GucSource source)
+{
+#ifdef HAVE_LIBCURL
+	return true;
+#else
+
+	/* if libcurl is not installed, only accept false */
+	if (*newval)
+	{
+		GUC_check_errcode(ERRCODE_FEATURE_NOT_SUPPORTED);
+		GUC_check_errdetail("Citus was compiled without libcurl support.");
+		return false;
+	}
+	else
+	{
+		return true;
+	}
+#endif
+}
