@@ -16,18 +16,22 @@
 
 #include "postgres.h"
 
+#include <time.h>
 
 #include "miscadmin.h"
 #include "pgstat.h"
 
 #include "access/xact.h"
 #include "catalog/pg_extension.h"
+#include "citus_version.h"
 #include "commands/extension.h"
 #include "libpq/pqsignal.h"
 #include "catalog/namespace.h"
 #include "distributed/distributed_deadlock_detection.h"
 #include "distributed/maintenanced.h"
+#include "distributed/master_protocol.h"
 #include "distributed/metadata_cache.h"
+#include "distributed/statistics_collection.h"
 #include "nodes/makefuncs.h"
 #include "postmaster/bgworker.h"
 #include "storage/ipc.h"
@@ -36,6 +40,7 @@
 #include "storage/lmgr.h"
 #include "storage/lwlock.h"
 #include "tcop/tcopprot.h"
+#include "utils/memutils.h"
 
 
 /*
@@ -209,6 +214,9 @@ CitusMaintenanceDaemonMain(Datum main_arg)
 {
 	Oid databaseOid = DatumGetObjectId(main_arg);
 	MaintenanceDaemonDBData *myDbData = NULL;
+	TimestampTz nextStatsCollectionTime USED_WITH_LIBCURL_ONLY =
+		TimestampTzPlusMilliseconds(GetCurrentTimestamp(), 60 * 1000);
+	bool retryStatsCollection USED_WITH_LIBCURL_ONLY = false;
 	ErrorContextCallback errorCallback;
 
 	/*
@@ -275,13 +283,13 @@ CitusMaintenanceDaemonMain(Datum main_arg)
 		CHECK_FOR_INTERRUPTS();
 
 		/*
-		 * XXX: We clear the metadata cache before every iteration because otherwise
-		 * it might contain stale OIDs. It appears that in some cases invalidation
-		 * messages for a DROP EXTENSION may arrive during deadlock detection and
+		 * XXX: Each task should clear the metadata cache before every iteration
+		 * by calling InvalidateMetadataSystemCache(), because otherwise it
+		 * might contain stale OIDs. It appears that in some cases invalidation
+		 * messages for a DROP EXTENSION may arrive during these tasks and
 		 * this causes us to cache a stale pg_dist_node OID. We'd actually expect
 		 * all invalidations to arrive after obtaining a lock in LockCitusExtension.
 		 */
-		InvalidateMetadataSystemCache();
 
 		/*
 		 * Perform Work.  If a specific task needs to be called sooner than
@@ -289,9 +297,72 @@ CitusMaintenanceDaemonMain(Datum main_arg)
 		 * tasks should do their own time math about whether to re-run checks.
 		 */
 
+#ifdef HAVE_LIBCURL
+		if (EnableStatisticsCollection &&
+			GetCurrentTimestamp() >= nextStatsCollectionTime)
+		{
+			bool statsCollectionSuccess = false;
+			InvalidateMetadataSystemCache();
+			StartTransactionCommand();
+
+			/*
+			 * Lock the extension such that it cannot be dropped or created
+			 * concurrently. Skip statistics collection if citus extension is
+			 * not accessible.
+			 *
+			 * Similarly, we skip statistics collection if there exists any
+			 * version mismatch or the extension is not fully created yet.
+			 */
+			if (!LockCitusExtension())
+			{
+				ereport(DEBUG1, (errmsg("could not lock the citus extension, "
+										"skipping statistics collection")));
+			}
+			else if (CheckCitusVersion(DEBUG1) && CitusHasBeenLoaded())
+			{
+				FlushDistTableCache();
+				WarnIfSyncDNS();
+				statsCollectionSuccess = CollectBasicUsageStatistics();
+				if (statsCollectionSuccess)
+				{
+					/*
+					 * Checking for updates only when collecting statistics succeeds,
+					 * so we don't log update messages twice when we retry statistics
+					 * collection in a minute.
+					 */
+					CheckForUpdates();
+				}
+			}
+
+			/*
+			 * If statistics collection was successful the next collection is
+			 * 24-hours later. Also, if this was a retry attempt we don't do
+			 * any more retries until 24-hours later, so we limit number of
+			 * retries to one.
+			 */
+			if (statsCollectionSuccess || retryStatsCollection)
+			{
+				nextStatsCollectionTime =
+					TimestampTzPlusMilliseconds(GetCurrentTimestamp(),
+												STATS_COLLECTION_TIMEOUT_MILLIS);
+				retryStatsCollection = false;
+			}
+			else
+			{
+				nextStatsCollectionTime =
+					TimestampTzPlusMilliseconds(GetCurrentTimestamp(),
+												STATS_COLLECTION_RETRY_TIMEOUT_MILLIS);
+				retryStatsCollection = true;
+			}
+
+			CommitTransactionCommand();
+		}
+#endif
+
 		/* the config value -1 disables the distributed deadlock detection  */
 		if (DistributedDeadlockDetectionTimeoutFactor != -1.0)
 		{
+			InvalidateMetadataSystemCache();
 			StartTransactionCommand();
 
 			/*
