@@ -31,6 +31,7 @@
 #include "distributed/metadata_cache.h"
 #include "distributed/multi_logical_optimizer.h"
 #include "distributed/pg_dist_local_group.h"
+#include "distributed/pg_dist_node_metadata.h"
 #include "distributed/pg_dist_node.h"
 #include "distributed/pg_dist_partition.h"
 #include "distributed/pg_dist_shard.h"
@@ -99,6 +100,7 @@ typedef struct MetadataCacheData
 	Oid distShardRelationId;
 	Oid distPlacementRelationId;
 	Oid distNodeRelationId;
+	Oid distNodeNodeIdIndexId;
 	Oid distLocalGroupRelationId;
 	Oid distColocationRelationId;
 	Oid distColocationConfigurationIndexId;
@@ -175,6 +177,7 @@ static void RegisterWorkerNodeCacheCallbacks(void);
 static void RegisterLocalGroupIdCacheCallbacks(void);
 static uint32 WorkerNodeHashCode(const void *key, Size keySize);
 static void ResetDistTableCacheEntry(DistTableCacheEntry *cacheEntry);
+static void CreateDistTableCache(void);
 static void InvalidateDistRelationCacheCallback(Datum argument, Oid relationId);
 static void InvalidateNodeRelationCacheCallback(Datum argument, Oid relationId);
 static void InvalidateLocalGroupIdRelationCacheCallback(Datum argument, Oid relationId);
@@ -410,7 +413,7 @@ LoadShardPlacement(uint64 shardId, uint64 placementId)
 
 /*
  * FindShardPlacementOnGroup returns the shard placement for the given shard
- * on the given group, or returns NULL of no placement for the shard exists
+ * on the given group, or returns NULL if no placement for the shard exists
  * on the group.
  */
 ShardPlacement *
@@ -1654,6 +1657,17 @@ DistNodeRelationId(void)
 }
 
 
+/* return oid of pg_dist_node's primary key index */
+Oid
+DistNodeNodeIdIndexId(void)
+{
+	CachedRelationLookup("pg_dist_node_pkey",
+						 &MetadataCache.distNodeNodeIdIndexId);
+
+	return MetadataCache.distNodeNodeIdIndexId;
+}
+
+
 /* return oid of pg_dist_local_group relation */
 Oid
 DistLocalGroupIdRelationId(void)
@@ -2332,13 +2346,7 @@ InitializeDistTableCache(void)
 	DistShardScanKey[0].sk_attno = Anum_pg_dist_shard_logicalrelid;
 
 	/* initialize the per-table hash table */
-	MemSet(&info, 0, sizeof(info));
-	info.keysize = sizeof(Oid);
-	info.entrysize = sizeof(DistTableCacheEntry);
-	info.hash = tag_hash;
-	DistTableCacheHash =
-		hash_create("Distributed Relation Cache", 32, &info,
-					HASH_ELEM | HASH_FUNCTION);
+	CreateDistTableCache();
 
 	/* initialize the per-shard hash table */
 	MemSet(&info, 0, sizeof(info));
@@ -2718,6 +2726,43 @@ InvalidateDistRelationCacheCallback(Datum argument, Oid relationId)
 }
 
 
+/*
+ * FlushDistTableCache flushes the entire distributed relation cache, frees
+ * all entries, and recreates the cache.
+ */
+void
+FlushDistTableCache(void)
+{
+	DistTableCacheEntry *cacheEntry = NULL;
+	HASH_SEQ_STATUS status;
+
+	hash_seq_init(&status, DistTableCacheHash);
+
+	while ((cacheEntry = (DistTableCacheEntry *) hash_seq_search(&status)) != NULL)
+	{
+		ResetDistTableCacheEntry(cacheEntry);
+	}
+
+	hash_destroy(DistTableCacheHash);
+	CreateDistTableCache();
+}
+
+
+/* CreateDistTableCache initializes the per-table hash table */
+static void
+CreateDistTableCache(void)
+{
+	HASHCTL info;
+	MemSet(&info, 0, sizeof(info));
+	info.keysize = sizeof(Oid);
+	info.entrysize = sizeof(DistTableCacheEntry);
+	info.hash = tag_hash;
+	DistTableCacheHash =
+		hash_create("Distributed Relation Cache", 32, &info,
+					HASH_ELEM | HASH_FUNCTION);
+}
+
+
 /*
  * InvalidateMetadataSystemCache resets all the cached OIDs and the extensionLoaded flag,
  * and invalidates the worker node and local group ID caches.
@@ -3155,3 +3200,51 @@ CitusInvalidateRelcacheByShardId(int64 shardId)
 	/* bump command counter, to force invalidation to take effect */
 	CommandCounterIncrement();
 }
+
+
+/*
+ * DistNodeMetadata returns the single metadata jsonb object stored in
+ * pg_dist_node_metadata.
+ */
+Datum
+DistNodeMetadata(void)
+{
+	Datum metadata = 0;
+	SysScanDesc scanDescriptor = NULL;
+	ScanKeyData scanKey[1];
+	const int scanKeyCount = 0;
+	HeapTuple heapTuple = NULL;
+	Oid metadataTableOid = InvalidOid;
+	Relation pgDistNodeMetadata = NULL;
+	TupleDesc tupleDescriptor = NULL;
+
+	metadataTableOid = get_relname_relid("pg_dist_node_metadata", PG_CATALOG_NAMESPACE);
+	if (metadataTableOid == InvalidOid)
+	{
+		ereport(ERROR, (errmsg("pg_dist_node_metadata was not found")));
+	}
+
+	pgDistNodeMetadata = heap_open(metadataTableOid, AccessShareLock);
+	scanDescriptor = systable_beginscan(pgDistNodeMetadata,
+										InvalidOid, false,
+										NULL, scanKeyCount, scanKey);
+	tupleDescriptor = RelationGetDescr(pgDistNodeMetadata);
+
+	heapTuple = systable_getnext(scanDescriptor);
+	if (HeapTupleIsValid(heapTuple))
+	{
+		bool isNull = false;
+		metadata = heap_getattr(heapTuple, Anum_pg_dist_node_metadata_metadata,
+								tupleDescriptor, &isNull);
+		Assert(!isNull);
+	}
+	else
+	{
+		ereport(ERROR, (errmsg("could not find any entries in pg_dist_metadata")));
+	}
+
+	systable_endscan(scanDescriptor);
+	heap_close(pgDistNodeMetadata, AccessShareLock);
+
+	return metadata;
+}
