@@ -35,6 +35,7 @@
 #include "distributed/resource_lock.h"
 #include "distributed/transmit.h"
 #include "distributed/worker_protocol.h"
+#include "distributed/version_compat.h"
 #include "executor/spi.h"
 #include "mb/pg_wchar.h"
 #include "storage/lmgr.h"
@@ -191,7 +192,7 @@ worker_hash_partition_table(PG_FUNCTION_ARGS)
 	CheckCitusVersion(ERROR);
 
 	/* use column's type information to get the hashing function */
-	hashFunction = GetFunctionInfo(partitionColumnType, HASH_AM_OID, HASHPROC);
+	hashFunction = GetFunctionInfo(partitionColumnType, HASH_AM_OID, HASHSTANDARD_PROC);
 
 	/* create hash partition context object */
 	partitionContext = palloc0(sizeof(HashPartitionContext));
@@ -409,7 +410,7 @@ OpenPartitionFiles(StringInfo directoryName, uint32 fileCount)
 	{
 		StringInfo filePath = PartitionFilename(directoryName, fileIndex);
 
-		fileDescriptor = PathNameOpenFile(filePath->data, fileFlags, fileMode);
+		fileDescriptor = PathNameOpenFilePerm(filePath->data, fileFlags, fileMode);
 		if (fileDescriptor < 0)
 		{
 			ereport(ERROR, (errcode_for_file_access(),
@@ -868,7 +869,7 @@ FilterAndPartitionTable(const char *filterQuery,
 			heap_deform_tuple(row, rowDescriptor, valueArray, isNullArray);
 
 			AppendCopyRowData(valueArray, isNullArray, rowDescriptor,
-							  rowOutputState, columnOutputFunctions);
+							  rowOutputState, columnOutputFunctions, NULL);
 
 			rowText = rowOutputState->fe_msgbuf;
 
