@@ -86,7 +86,9 @@ extern List * DistTableOidList(void);
 extern List * ShardPlacementList(uint64 shardId);
 extern void CitusInvalidateRelcacheByRelid(Oid relationId);
 extern void CitusInvalidateRelcacheByShardId(int64 shardId);
+extern void FlushDistTableCache(void);
 extern void InvalidateMetadataSystemCache(void);
+extern Datum DistNodeMetadata(void);
 
 extern bool CitusHasBeenLoaded(void);
 extern bool CheckCitusVersion(int elevel);
@@ -109,6 +111,7 @@ extern Oid DistNodeRelationId(void);
 extern Oid DistLocalGroupIdRelationId(void);
 
 /* index oids */
+extern Oid DistNodeNodeIdIndexId(void);
 extern Oid DistPartitionLogicalRelidIndexId(void);
 extern Oid DistPartitionColocationidIndexId(void);
 extern Oid DistShardLogicalRelidIndexId(void);
