@@ -1,3 +1,15 @@
+------------------------------------
+------------------------------------
+-- Insert into local table
+------------------------------------
+------------------------------------
+CREATE TABLE test_table_1(id int);
+
+INSERT INTO test_table_1 
+SELECT user_id FROM users_table;
+
+DROP TABLE test_table_1;
+
 ------------------------------------
 ------------------------------------
 -- Vanilla funnel query
