@@ -138,14 +138,6 @@ SELECT master_remove_node('localhost', 5432);
 TRUNCATE mx_table;
 SELECT count(*) FROM mx_table;
 
--- INSERT / SELECT pulls results to worker
-BEGIN;
-SET LOCAL client_min_messages TO DEBUG;
-INSERT INTO mx_table_2 SELECT * FROM mx_table;
-END;
-
-SELECT count(*) FROM mx_table_2;
-
 -- mark_tables_colocated
 UPDATE pg_dist_partition SET colocationid = 0 WHERE logicalrelid='mx_table_2'::regclass;
 
